// Test driver for strtree.h/.cpp

#include <iostream.h>
#include "strtree.h"
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <string.h>

char *Tests[] =
{
	"",
	"the",
	"of",
	"and",
	"to",
	"a",
	"in",
	"I",
	"that",
	"at",
	"int",
	"into",
	"insane",
	"instance",
	"i)",
	"ii)",
	"I",
	"I",
	"i)",
	"www.test.com",
	"www.test.com/index.html",
	"www.test.com",
	"www.test.com/index2.html",
	"www.site.org/index.html",
	"www.another.site.org/index.html",
	"www.site.another.org/index.html",
	"www.test.com/index.htm",
	"www.",
	"123456",
	"12345",
	"1234567",
	"123456",
	"12345678",
	"1234567",
	"1234567890",
	"www.test.com/index.html",
	"123456789",
	"12345678",
	"1234",
	"12345",
	"ywtwvwmbyyqxeimgeeajk",
};

const int iNumTests = sizeof(Tests) / sizeof(Tests[0]);

enum
{
	LINEAR,
	ST_UNSORTED,
	ST_SORTED
};


int main()
{
#ifndef __BORLANDC__
	_timeb last, current;
#else
	timeb last, current;
#endif

	char szBuf[255];
	int iStored = 0;
	int iBaseReps = 15000;
	int iReps;

	// Construct the tree
	CStringTree StringTree;
	
	// Sort the items as they are added
	StringTree.EnableSorting(true);
	

	// Allocate some storage for a crude linear search
	char **szStorage = new char*[iBaseReps];
	for(int init = 0; init < iBaseReps; init++)
	{
		szStorage[init] = new char[255];
	}

	// Generate a random, but reuseable seed
	cout << "Randomizing..." << endl;
	time_t iRandSeed = time(NULL);
	
	// Test and verify simple operation
	cout << "Testing insertion of manual test cases..." << endl;
	for(int i = 0; i < iNumTests; i++)
	{
		if(StringTree.AddString(Tests[i]))
			cout << "+ " << Tests[i] << endl;
		else
			cout << "- " << Tests[i] << endl;
		if(!StringTree.Find(Tests[i]))
		{
			cout << "Find() failed to locate " << Tests[i] << endl;
			StringTree.Find(Tests[i]);
		}
		char szUC[255];
		strcpy(szUC, Tests[i]);

#ifndef __BORLANDC__
		_strupr(szUC);
#else
		strupr(szUC);
#endif

		if(!StringTree.Find(szUC, false))
		{
			cout << "Case insensitive search for " << szUC << " failed." << endl;
			StringTree.Find(szUC, false);
		}

	}

	cout << endl;
	cout << "Dumping raw contents of tree..." << endl;
	StringTree.Dump();

	cout << endl;
	cout << "Dumping tree-view of tree..." << endl;
	StringTree.Display();
	cout << endl;

	StringTree.Purge();
	
	FILE *f = fopen("tests.txt", "r");
	if(f)
	{
		char szRead[255];
		cout << "Testing insertion of manual test cases..." << endl;
		while(!feof(f))
		{
			fscanf(f, "%s", szRead);
			StringTree.AddString(szRead);
			if(!StringTree.Find(szRead))
			{
				cout << "Find() failed to locate " << szRead << endl;
				StringTree.Find(szRead);
			}
			char szUC[255];
			strcpy(szUC, szRead);

#ifndef __BORLANDC__
		_strupr(szUC);
#else
		strupr(szUC);
#endif

			if(!StringTree.Find(szUC, false))
			{
				cout << "Case insensitive search for " << szUC << " failed." << endl;
				StringTree.Find(szUC, false);
			}

		}
		fclose(f);
	}
	// Display the tree data in variious formats
	cout << endl;
	cout << "Dumping raw contents of tree..." << endl;
	StringTree.Dump();

	// Empty the tree and prep for extended testing
	cout.precision(2);
	
	for(int iCase = LINEAR; iCase <= ST_SORTED ; iCase++)
	{
		// Use the saved random key for each method
		cout << "Resetting random key" << endl;
		srand(iRandSeed);
		
		
		cout << endl;
		cout << "<Purging>" << endl;
		StringTree.Purge();

		// Because of the performance gains, it is useful to increase the number of test cases for the string tree approach
		// if(iCase != LINEAR)
		//     iReps = iBaseReps * 20
		iReps = iBaseReps;
		
		if(iCase == LINEAR)
		{
			StringTree.EnableSorting(false);
			cout << "LINEAR search used, sorting NOT AVAILABLE" << endl;
		}
		if(iCase == ST_UNSORTED)
		{
			StringTree.EnableSorting(false);
			cout << "CStringTree used, sorting DISABLED" << endl;
		}
		if(iCase == ST_SORTED)
		{
			StringTree.EnableSorting(true);
			cout << "CStringTree used, sorting ENABLED" << endl;
		}

		cout << "Beginning insertion of " << iReps << " strings of random lengths \nbetween 0 and 254 charaters in length" << endl;
		cout << "Total strings / Time elapsed during 3,000 additional insertions" << endl << endl;
		
		// Record the starting time
#ifndef __BORLANDC__
		_ftime(&last);
#else
		ftime(&last);
#endif
		
		// Keep track of how many strings were rejected as duplicates, so we can verify that sorted and unsorted trees behave identically
		int iRejected = 0;
		// Generate random strings and use the appropriate algorithm to check for duplicates
		for(int y = 0; y <= iReps; y++)
		{
			// Randomize the length of each string
			int iLen = rand() % 25;
			
			// Fill with alpha characters
			for(int u = 0; u < iLen; u++)
				szBuf[u] = 'a' + rand() % 25;
			
			// NTS please!
			szBuf[iLen] = 0;

			// The linear search simply scans each string in it's history with a strcmp.
			// Not very efficient at all, but the simplest implementation
			if(iCase == LINEAR)
			{
				bool bUnique = true;
				for(int p = 0; p < iStored; p++)
					if(!strcmp(szStorage[p], szBuf))
					{
						bUnique = false;
						break;
					}
				if(bUnique)
					strcpy(szStorage[iStored++], szBuf);
				else
					iRejected++;
			}
			// The StringTree search uses an N-Tree to eliminate leading redundancy, 
			// and uniquely positions each string such that it can be found quickly 
			// if the exact string is known beforehand
			else
			{
				int iFound = StringTree.Find(szBuf);
				if(!szBuf[0])
					int iDebug = 1;
				if(!StringTree.AddString(szBuf))
				{
					if(!iFound)
					{
						cout << "Find() failed to locate " << szBuf << endl;
						StringTree.Find(szBuf);
					}
					iRejected++;
				}
				else
				{
					if(iFound)
					{
						cout << "Find() erroneously located " << szBuf << endl;
						StringTree.Find(szBuf);
					}
				}
			}

			// Output periodic statistics (simple, but it serves the purpose of demonstration)
			if(y && !(y % (iReps / 5)))
			{
#ifndef __BORLANDC__
				_ftime(&current);
#else
				ftime(&current);
#endif
				cout << y << "\t" << ((double) current.time + (double) current.millitm / 1000) - ((double) last.time + (double) last.millitm / 1000) << "\t seconds (" << iRejected << " duplicates)" << endl;
				last.time = current.time;
				last.millitm = current.millitm;
				iRejected = 0;
			}
		}
	}

	// Clean up
	for(init = 0; init < iBaseReps; init++)
		delete [] szStorage[init];
	delete [] szStorage;
	StringTree.Purge();

	// Ciao!
	return 0;
}

